%% Modeling a Differential High-Speed Backplane in Simulink and SimScape
% This demo shows how to use Simulink to simulate a differential high-speed
% backplane. The demo first reads a Touchstone data file that contains
% measured single-ended 4-port S-parameters for a differential high-speed
% backplane and converts them to 2-port differential S-parameters. It
% uses the RATIONALFIT function to fit a closed-form rational function model to the
% backplane's 4 differential s-parameters. 
% These rational functions are incoportated in a Simulink Model which 
% simulates a 2GBPS signal passing through the backplane.
% The key advantage of this technique is the load and source impedances in
% the Simulink/SimScape model can be varied without rebulding the
% s-parameter model. 
% Copyright 2006-2014 The Mathworks, Inc.
 
%% Read the Single-Ended 4-Port S-Parameters and 
% Convert Them to Differential 2-Port S-Parameters 
% Read a Touchstone data file into an RFDATA.DATA object using the object's
% READ method. The parameters in this data file are the 50-ohm S-parameters
% of a single-ended 4-port passive circuit, measured at 1496 frequencies
% ranging from 50 MHz to 15 GHz. Then, get the single-ended 4-port
% S-parameters from the data object, and use the matrix conversion
% function, S2SDD, to convert them to differential 2-port S-parameters.
clear
FileName = 'default.s4p';
SingleEndedData = read(rfdata.data, FileName);
SingleEndedSparams = SingleEndedData.S_Parameters;
Freq = SingleEndedData.Freq;
Zo=50;
X = s2sdd(SingleEndedSparams); % X contains the differntial s-parameters 

%% Fit a rational function in the polynomial Laplace variable S
% to each of these 4 s-parameters
for k=1:2
    for n=1:2

    if (k+n)==3
       DelayFactor=0.95; % s21 and s12
       npoles=[10 35];
       FittingTolerance = -35;   % Rational fitting tolerance in dB 
       
    else
       DelayFactor=0.0;  % s11 and s22
       npoles=[92,92];
       FittingTolerance = -30;   % Rational fitting tolerance in dB 
    end;

     S_Parameter_Rational_Functions(k,n) = ...
         rationalfit(Freq,X(k,n,:),FittingTolerance, 'DelayFactor',DelayFactor,'NPoles', npoles,'WaitBar',true )
    nPoles = length(S_Parameter_Rational_Functions(k,n).A);
    % Plot freq responses for a sanity check 
    % Phase may not be in 100% agreement since the unwrapping can be fooled
    % by noise. 
    figure;     % one figure per parameter 
    PlotFreq = Freq;
    RespAtPlotFrequency = freqresp(S_Parameter_Rational_Functions(k,n), PlotFreq);
    subplot(2,1,1); plot(Freq*1.e-9, 20*log10(abs(squeeze(X(k,n,:)))), 'r', ...
    PlotFreq*1.e-9, 20*log10(abs(RespAtPlotFrequency)), 'b--', 'LineWidth', 2);
    title(['Rational Fitting S(', num2str(k),' ',num2str(n),') ',num2str(nPoles),' Poles'], 'FontSize', 12);
    axis([0,PlotFreq(end)*1e-9,-40,0])
    ylabel('Magnitude (decibels)'); xlabel('Frequency (GHz)');
    legend('Original', 'Fitting result');
    subplot(2,1,2); plot(Freq*1.e-9, unwrap(angle(squeeze(X(k,n,:))))*180/pi, 'r', ...
    PlotFreq*1.e-9, unwrap(angle(RespAtPlotFrequency))*180/pi, 'b--', 'LineWidth', 2);
    ylabel('Phase in degrees'); xlabel('Frequency (GHz)');
    legend('Original', 'Fitting result');
    set(gcf,'Name','MATLAB Model Results');
    drawnow
    end
end
% save this to a file for possible use by other models
save s_parameter_rational_functions S_Parameter_Rational_Functions Zo

%% Simulate in Simulink
ModelName = 'S_test_backplane_SS_2';
open_system(ModelName);
% sim(ModelName)
